@@ -28,6 +28,7 @@
 /* not a VZ-capable kernel */
 #define VZ_BAD_KERNEL			5
 #define VZ_RESOURCE_ERROR		6
+/* #define VZCTL_E_NOMEM			VZ_RESOURCE_ERROR */
 #define VZ_ENVCREATE_ERROR		7
 
 #define VZ_COMMAND_EXECUTION_ERROR	8
@@ -164,7 +165,8 @@
 #define VZCTL_E_DELETE_SNAPSHOT		158
 #define VZCTL_E_SWITCH_SNAPSHOT		159
 #define VZCTL_E_MOUNT_SNAPSHOT		160
-#define VZCTL_E_COMPACT_IMAGE		161
-#define VZCTL_E_LIST_SNAPSHOT		162
+#define VZCTL_E_UMOUNT_SNAPSHOT		161
+#define VZCTL_E_COMPACT_IMAGE		166
+#define VZCTL_E_LIST_SNAPSHOT		167
 
 #endif /* _VZ_ERROR_H_ */
