@@ -31,6 +31,7 @@
 #include <sys/mount.h>
 #include <sys/utsname.h>
 #include <sys/stat.h>
+#include <sys/vfs.h>
 
 #include "vzerror.h"
 #include "res.h"
@@ -47,6 +48,10 @@
 #include "readelf.h"
 #include "destroy.h"
 
+#ifndef PROC_SUPER_MAGIC
+#define PROC_SUPER_MAGIC	0x9fa0
+#endif
+
 static int env_stop(vps_handler *h, envid_t veid, const char *root,
 		int stop_mode, int timeout);
 
@@ -218,7 +223,7 @@ void fill_container_param(struct arg_start *arg,
 			 struct env_create_param3 *create_param)
 {
 	memset(create_param, 0, sizeof(*create_param));
-	create_param->iptables_mask = get_ipt_mask(arg->res->env.ipt_mask);
+	create_param->iptables_mask = get_ipt_mask(&arg->res->env);
 	logger(3, 0, "Setting iptables mask %#10.8llx",
 			(unsigned long long) create_param->iptables_mask);
 	clean_hardlink_dir("/");
@@ -244,6 +249,7 @@ int exec_container_init(struct arg_start *arg,
 	int fd, ret;
 	char *argv[] = {"init", "-z", "      ", NULL};
 	char *envp[] = {"HOME=/", "TERM=linux", NULL};
+	struct statfs sfs;
 
 	/* Clear supplementary group IDs */
 	setgroups(0, NULL);
@@ -262,7 +268,12 @@ int exec_container_init(struct arg_start *arg,
 		}
 	}
 
-	if (access("/proc", F_OK) == 0 && mount("proc", "/proc", "proc", 0, 0))
+	if (statfs("/proc", &sfs))
+		return vzctl_err(VZ_SYSTEM_ERROR, errno,
+				"statfs on /proc failed");
+
+	if (sfs.f_type != PROC_SUPER_MAGIC &&
+	    mount("proc", "/proc", "proc", 0, 0))
 		return vzctl_err(VZ_SYSTEM_ERROR, errno,
 				"Failed to mount /proc");
 
