@@ -10,6 +10,7 @@
 #include <fcntl.h>
 #include <sched.h>
 #include <dirent.h>
+#include <sys/vfs.h>
 
 #include "vzerror.h"
 #include "env.h"
@@ -103,13 +104,23 @@ int ct_chroot(const char *root)
 	char oldroot[] = "vzctl-old-root.XXXXXX";
 	int ret = VZ_RESOURCE_ERROR;
 
-	if (chdir(root)) {
-		logger(-1, errno, "Can't chdir %s", root);
+	if (mount("", "/", NULL, MS_PRIVATE|MS_REC, NULL) < 0) {
+		logger(-1, errno, "Can't remount root with MS_PRIVATE");
 		return ret;
 	}
 
-	if (mount("", "/", NULL, MS_PRIVATE|MS_REC, NULL) < 0) {
-		logger(-1, errno, "Can't remount root with MS_PRIVATE");
+	/* root must be bind-mounted to itself to not show what is under it
+	 *
+	 * Linux kernel commit 5ff9d8a6
+	 * "vfs: Lock in place mounts from more privileged users"
+	 */
+	if (mount(root, root, NULL, MS_BIND | MS_REC, NULL)) {
+		logger(-1, errno, "Can't bind-mount root %s", root);
+		return ret;
+	}
+
+	if (chdir(root)) {
+		logger(-1, errno, "Can't chdir %s", root);
 		return ret;
 	}
 
@@ -128,6 +139,21 @@ int ct_chroot(const char *root)
 		goto rmdir;
 	}
 
+	/* proc and sysfs must be mounted before unmounting oldroot
+	 * because of Linux kernel commit e51db7
+	 * "userns: Better restrictions on when proc and sysfs can be mounted"
+	 */
+
+	if (mount("proc", "/proc", "proc", 0, 0)) {
+		logger(-1, errno, "Failed to mount /proc");
+		goto rmdir;
+	}
+
+	if (mount("sysfs", "/sys", "sysfs", 0, 0)) {
+		logger(-1, errno, "Failed to mount /sys");
+		goto rmdir;
+	}
+
 	if (umount2(oldroot, MNT_DETACH)) {
 		logger(-1, 0, "Can't umount old mounts");
 		goto rmdir;
@@ -244,94 +270,6 @@ out:
 	return ret;
 }
 
-/*
- * Those devices should exist in the container, and be valid device nodes with
- * user access permission. But we need to be absolutely sure this is the case,
- * so we will provide our own versions. That could actually happen since some
- * distributions may come with emptied /dev's, waiting for udev to populate them.
- * That won't happen, we do it ourselves.
- */
-static void create_devices(vps_handler *h, envid_t veid, const char *root)
-{
-	unsigned int i;
-	char *devices[] = {
-		"/dev/null",
-		"/dev/zero",
-		"/dev/random",
-		"/dev/urandom",
-	};
-
-	/*
-	 * We will tolerate errors, and keep the container running, because it is
-	 * likely we will be able to boot it to a barely functional state. But
-	 * be vocal about it
-	 */
-	for (i = 0; i < ARRAY_SIZE(devices); i++) {
-		char ct_devname[STR_SIZE];
-		int ret;
-
-		snprintf(ct_devname, sizeof(ct_devname), "%s%s", root, devices[i]);
-
-		/*
-		 * No need to be crazy about file flags. When we bind mount, the
-		 * source permissions will be inherited.
-		 */
-		ret = open(ct_devname, O_RDWR|O_CREAT, 0);
-		if (ret < 0) {
-			logger(-1, errno, "Could not touch device %s", devices[i]);
-			continue;
-		}
-		close(ret);
-
-		ret = mount(devices[i], ct_devname, "", MS_BIND, 0);
-		if (ret < 0)
-			logger(-1, errno, "Could not bind mount device %s", devices[i]);
-	}
-}
-
-static int mount_devpts(void)
-{
-	int fd, ret;
-
-	setuid(0);
-	setgid(0);
-
-	ret = mkdir("/dev/pts", S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
-	if ((ret < 0) && (errno != EEXIST)) {
-		logger(-1, errno, "Cannot create container's /dev/pts");
-		return VZ_RESOURCE_ERROR;
-	}
-	ret = mount("devpts", "/dev/pts", "devpts", 0, "newinstance");
-	if (ret < 0) {
-		/* No need to cleanup mkdir, since we test for EEXIST */
-		logger(-1, errno, "Cannot mount container's /dev/pts");
-		return VZ_RESOURCE_ERROR;
-	}
-
-	/* /dev/ptmx, if it even exists, would refer to the root ptmx.
-	 * We don't want that, we want our newly created instance to contain
-	 * all ptys. So we bind mount the root device here
-	 */
-	fd = open("/dev/ptmx", O_CREAT, S_IRWXU|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
-	if (fd < 0) {
-		logger(-1, errno, "Cannot create container's /dev/ptmx");
-		/*
-		 * No need to umount, we are in a private mnt namespace and it will
-		 * disappear after we fail.
-		 */
-		return VZ_RESOURCE_ERROR;
-	}
-	close(fd);
-	ret = mount("/dev/pts/ptmx", "/dev/ptmx", "", MS_BIND, 0);
-	if (ret < 0) {
-		/* No need to cleanup mkdir, since we test for EEXIST */
-		logger(-1, errno, "Cannot bind mount container's /dev/ptmx");
-		return VZ_RESOURCE_ERROR;
-	}
-
-	return 0;
-}
-
 static int _env_create(void *data)
 {
 	struct arg_start *arg = data;
@@ -356,18 +294,13 @@ static int _env_create(void *data)
 	if (arg->userns_p != -1)
 		close(arg->userns_p);
 
-	if (arg->h->can_join_userns) {
-		create_devices(arg->h, arg->veid, arg->res->fs.root);
-	}
-
 	ret = ct_chroot(arg->res->fs.root);
 	/* Probably means chroot failed */
 	if (ret)
 		return ret;
 
-	ret = mount_devpts();
-	if (ret)
-		return ret;
+	setuid(0);
+	setgid(0);
 
 	/*
 	 * If we are using the user namespace, we will have the full capability
@@ -419,11 +352,25 @@ static int ct_env_create_real(struct arg_start *arg)
 
 		userns_p[0] = userns_p[1] = -1;
 	} else {
+		char devpath[PATH_MAX];
+
 		clone_flags |= CLONE_NEWUSER;
 		if (pipe(userns_p) < 0) {
 			logger(-1, errno, "Can not create userns pipe");
 			return VZ_RESOURCE_ERROR;
 		}
+
+		/* Unshare mntns to not affect the host system */
+		if (unshare(CLONE_NEWNS)) {
+			logger(-1, errno, "Can not unshare mount namespace");
+			return VZ_RESOURCE_ERROR;
+		}
+
+		snprintf(devpath, sizeof(devpath), "%s/dev", arg->res->fs.root);
+		if (mount("dev", devpath, "devtmpfs", 0, NULL)) {
+			logger(-1, errno, "Can not mount devtmpfs");
+			return VZ_RESOURCE_ERROR;
+		}
 	}
 	arg->userns_p = userns_p[0];
 
