@@ -505,7 +505,10 @@ static int setup_hardlink_dir(const char *mntdir, int cpt_fd)
 	int fd, res = 0;
 
 	snprintf(buf, sizeof(buf), "%s/%s", mntdir, CPT_HARDLINK_DIR);
-	mkdir(buf, 0711);
+	if (mkdir(buf, 0711) && errno != EEXIST)
+		return vzctl_err(VZ_SYSTEM_ERROR, errno,
+				"Unable to create hardlink directory %s",
+				buf);
 
 	fd = open(buf, O_RDONLY | O_NOFOLLOW | O_DIRECTORY);
 	if (fd < 0) {
@@ -744,8 +747,11 @@ static int restore_fn(vps_handler *h, envid_t veid, const vps_res *res,
 	close(error_pipe[1]);
 
 	ret = ioctl(param->rst_fd, CPT_SET_LOCKFD2, wait_p);
-	if (ret < 0 && errno == EINVAL)
+	if (ret < 0 && errno == EINVAL) {
+		logger(0, 0, "Warning: old kernel -- CPT_SET_LOCKFD2 "
+				"not supported");
 		ret = ioctl(param->rst_fd, CPT_SET_LOCKFD, old_wait_p);
+	}
 	if (ret < 0) {
 		logger(-1, errno, "Can't set lockfd");
 		goto err;
