@@ -180,11 +180,13 @@ out:
 }
 
 int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
-		vps_param *g_p,	const fs_param *fs, const char *guid)
+		vps_param *g_p, const struct vzctl_snapshot_param *param)
 {
 	int ret, run;
 	int flags = 0;
 	cpt_param cpt = {};
+	const char *guid = param->guid;
+	fs_param *fs = &g_p->res.fs;
 	char fname[PATH_MAX];
 	char snap_xml_tmp[PATH_MAX];
 	char ve_conf_tmp[PATH_MAX] = "";
@@ -193,7 +195,7 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	const char *guid_tmp = NULL;
 	struct vzctl_snapshot_tree *tree = NULL;
 	struct ploop_disk_images_data *di = NULL;
-	struct ploop_snapshot_switch_param param = {};
+	struct ploop_snapshot_switch_param switch_param = {};
 
 	if (!is_snapshot_supported(fs->private))
 		return VZCTL_E_SWITCH_SNAPSHOT;
@@ -253,16 +255,16 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	}
 
 	/* switch snapshot */
-	param.guid = guid;
-	param.guid_old = guid_tmp;
-	param.flags = flags;
-	PLOOP_CLEANUP(ret = ploop.switch_snapshot_ex(di, &param));
+	switch_param.guid = guid;
+	switch_param.guid_old = guid_tmp;
+	switch_param.flags = flags;
+	PLOOP_CLEANUP(ret = ploop.switch_snapshot_ex(di, &switch_param));
 	if (ret)
 		goto err2;
 
 	/* restore ve.conf */
 	vzctl_get_snapshot_ve_conf(fs->private, guid, fname, sizeof(fname));
-	if (stat_file(fname) == 1) {
+	if (stat_file(fname) == 1 && !(param->flags & SNAPSHOT_SKIP_CONFIG)) {
 		get_vps_conf_path(veid, ve_conf_tmp, sizeof(ve_conf_tmp) - 4);
 		strcat(ve_conf_tmp, ".tmp");
 		if (cp_file(ve_conf_tmp, fname))
@@ -285,10 +287,13 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	}
 	/* resume CT in case dump file exists (no rollback, ignore error) */
 	vzctl_get_snapshot_dumpfile(fs->private, guid, dumpfile, sizeof(dumpfile));
-	if (stat_file(dumpfile) == 1) {
+	if (stat_file(dumpfile) == 1 &&	!(param->flags & SNAPSHOT_SKIP_RESUME)) {
 		vps_param *param;
 
 		param = reread_vps_config(veid);
+		if (param && g_p->res.net.skip_arpdetect)
+			param->res.net.skip_arpdetect =
+				g_p->res.net.skip_arpdetect;
 		cpt.dumpfile = dumpfile;
 		if (vps_restore(h, veid, (param) ? param : g_p,
 					CMD_RESTORE, &cpt, SKIP_DUMPFILE_UNLINK))
@@ -310,10 +315,10 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 
 err3:
 	if (guid_tmp != NULL) {
-		param.guid = guid_tmp;
-		param.guid_old = NULL;
-		param.flags = PLOOP_SNAP_SKIP_TOPDELTA_CREATE;
-		PLOOP_CLEANUP(ploop.switch_snapshot_ex(di, &param));
+		switch_param.guid = guid_tmp;
+		switch_param.guid_old = NULL;
+		switch_param.flags = PLOOP_SNAP_SKIP_TOPDELTA_CREATE;
+		PLOOP_CLEANUP(ploop.switch_snapshot_ex(di, &switch_param));
 	}
 
 err2:
