@@ -60,6 +60,7 @@ static struct option set_opt[] = {
 			required_argument, NULL, PARAM_APPLYCONFIG_MAP},
 	{"reset_ub",	no_argument, NULL, PARAM_RESET_UB},
 	{"iptables",	required_argument, NULL, PARAM_IPTABLES},
+	{"netfilter",	required_argument, NULL, PARAM_NETFILTER},
 	/*	UB	*/
 	{"kmemsize",	required_argument, NULL, PARAM_KMEMSIZE},
 	{"lockedpages",	required_argument, NULL, PARAM_LOCKEDPAGES},
@@ -153,6 +154,9 @@ static struct option set_opt[] = {
 	{"swap",	required_argument, NULL, PARAM_SWAP},
 
 	{"stop-timeout", required_argument, NULL, PARAM_STOP_TIMEOUT},
+	{"offline-resize",  no_argument, NULL, PARAM_OFFLINE_RESIZE},
+		/* Alias for PVC compatibility */
+		{"offline", no_argument, NULL, PARAM_OFFLINE_RESIZE},
 
 	{NULL, 0, NULL, 0}
 };
@@ -407,6 +411,7 @@ static int parse_create_opt(envid_t veid, int argc, char **argv,
 	{"ve_layout",	required_argument, NULL, PARAM_VE_LAYOUT},
 	{"velayout",	required_argument, NULL, PARAM_VE_LAYOUT},
 	{"diskspace",	required_argument, NULL, PARAM_DISKSPACE},
+	{"diskinodes",	required_argument, NULL, PARAM_DISKINODES},
 	{"local_uid",	required_argument, NULL, PARAM_LOCAL_UID},
 	{"local_gid",	required_argument, NULL, PARAM_LOCAL_GID},
 	{ NULL, 0, NULL, 0 }
@@ -734,6 +739,34 @@ static int parse_snapshot_delete_opt(envid_t veid, int argc, char **argv,
 	return ret;
 }
 
+static int parse_snapshot_switch_opt(envid_t veid, int argc, char **argv,
+		vps_param *param)
+{
+	int ret;
+	struct option *opt;
+	static struct option snapshot_switch_options[] =
+	{
+	{"id",  required_argument, NULL, PARAM_SNAPSHOT_GUID},
+	{"uuid", required_argument, NULL, PARAM_SNAPSHOT_GUID},
+	{"skip_arpdetect", no_argument, NULL, PARAM_SKIPARPDETECT},
+	{"skip-resume", no_argument, NULL, PARAM_SNAPSHOT_SKIP_RESUME},
+	{"skip-config", no_argument, NULL, PARAM_SNAPSHOT_SKIP_CONFIG},
+	{ NULL, 0, NULL, 0 }
+	};
+
+	opt = mod_make_opt(snapshot_switch_options, &g_action, NULL);
+	if (opt == NULL)
+		return VZ_RESOURCE_ERROR;
+	ret = parse_opt(veid, argc, argv, opt, snapshot_switch_options, param);
+	free(opt);
+
+	if (ret == 0 && param->snap.guid == NULL)
+		return vzctl_err(VZ_INVALID_PARAMETER_SYNTAX, 0,
+			"Invalid syntax: snapshot id is missing");
+
+	return ret;
+}
+
 static int parse_snapshot_mount_opt(envid_t veid, int argc, char **argv,
 		vps_param *param)
 {
@@ -813,11 +846,13 @@ static int check_set_mode(vps_handler *h, envid_t veid, int setmode, int apply,
 	/* If iptables mask is set and it differs from the old one.
 	 * FIXME: we don't catch the case when the new set is empty
 	 * and the old one is not (vzctl set --iptables '') */
-	if (new_res->env.ipt_mask &&
-			new_res->env.ipt_mask != old_res->env.ipt_mask)
+	if (	(new_res->env.ipt_mask &&
+			new_res->env.ipt_mask != old_res->env.ipt_mask) ||
+		(new_res->env.nf_mask &&
+			new_res->env.nf_mask  != old_res->env.nf_mask))
 	{
 		if (loud)
-			logger(-1, 0, "Unable to set iptables "
+			logger(-1, 0, "Unable to set iptables/netfilter "
 					"on a running container");
 		found++;
 	}
@@ -922,6 +957,19 @@ static int check_set_opt(int argc, char *argv[], vps_param *param)
 		return VZ_NOTENOUGHUBCPARAMS;
 	}
 
+	/* --offline-resize can only be used with --diskspace on ploop */
+	if (param->res.dq.offline_resize == YES) {
+		if (!param->res.dq.diskspace) {
+			logger(-1, 0, "Error: option --offline-resize "
+					"can only be used with --diskspace");
+			return VZ_INVALID_PARAMETER_SYNTAX;
+		}
+		if (!ve_private_is_ploop(param->g_param->res.fs.private)) {
+			logger(0, 0, "Warning: option --offline-resize "
+					"only makes sense for a ploop CT");
+		}
+	}
+
 	/* A few options require --save flag */
 	if (!param->opt.save) {
 		if (param->res.name.name != NULL) {
@@ -1143,9 +1191,25 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 			ret=VZ_PLOOP_UNSUP;
 			goto err;
 		}
+		/* offline resize requires unmounted container */
+		if (cmd_p->res.dq.offline_resize == YES) {
+			int is_mounted = (vps_is_mounted(g_p->res.fs.root,
+						g_p->res.fs.private) == 1);
+
+			if (is_run || is_mounted) {
+				logger(-1, 0, "Option --offline-resize "
+						"can't be used on a "
+						"%s container",
+						is_run ? "running"
+							: "mounted");
+				ret = VZ_INVALID_PARAMETER_VALUE;
+				goto err;
+			}
+		}
 #ifdef HAVE_PLOOP
 		if ((ret = vzctl_resize_image(g_p->res.fs.private,
-						cmd_p->res.dq.diskspace[1])))
+						cmd_p->res.dq.diskspace[1],
+						cmd_p->res.dq.offline_resize)))
 			goto err;
 #endif
 	}
@@ -1480,8 +1544,10 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	case ACTION_SNAPSHOT_CREATE:
 		ret = parse_snapshot_create_opt(veid, argc, argv, param);
 		break;
-	case ACTION_SNAPSHOT_DELETE:
 	case ACTION_SNAPSHOT_SWITCH:
+		ret = parse_snapshot_switch_opt(veid, argc, argv, param);
+		break;
+	case ACTION_SNAPSHOT_DELETE:
 	case ACTION_SNAPSHOT_UMOUNT:
 		ret = parse_snapshot_delete_opt(veid, argc, argv, param);
 		break;
@@ -1658,8 +1724,8 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 				&g_p->res.fs, cmd_p->snap.guid);
 		break;
 	case ACTION_SNAPSHOT_SWITCH:
-		ret = vzctl_env_switch_snapshot(h, veid, g_p, &g_p->res.fs,
-				cmd_p->snap.guid);
+		g_p->res.net.skip_arpdetect = cmd_p->res.net.skip_arpdetect;
+		ret = vzctl_env_switch_snapshot(h, veid, g_p, &cmd_p->snap);
 		break;
 	case ACTION_SNAPSHOT_LIST:
 		ret = vzctl_env_snapshot_list(argc, argv, veid,
